package com.chelvc.framework.security.interceptor;

import java.util.Collections;
import java.util.List;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.exception.FrameworkException;
import com.chelvc.framework.common.model.Ip;
import com.chelvc.framework.common.util.JacksonUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.redis.context.RedisContextHolder;
import com.chelvc.framework.security.firewall.FirewallProcessor;
import com.chelvc.framework.security.firewall.Rule;
import com.fasterxml.jackson.core.type.TypeReference;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.core.Ordered;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Component;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

/**
 * 安全防火墙拦截器
 *
 * @author Woody
 * @date 2025/2/23
 */
@Slf4j
@Component
@ConditionalOnClass(RedisContextHolder.class)
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class SecurityFirewallInterceptor implements HandlerInterceptor, WebMvcConfigurer {
    /**
     * 规则列表类型定义
     */
    private static final TypeReference<List<Rule>> RULE_LIST_TYPE = new TypeReference<List<Rule>>() {
    };

    private final FirewallProcessor processor;

    /**
     * 判断功能是否启用
     *
     * @return true/false
     */
    private boolean isEnabled() {
        return ApplicationContextHolder.getProperty("security.firewall.enabled", boolean.class, false);
    }

    /**
     * 获取规则配置
     *
     * @return 规则列表
     */
    private List<Rule> getRules() {
        return ApplicationContextHolder.getProperty("security.firewall.rules", json -> {
            List<Rule> rules = JacksonUtils.deserialize(json, RULE_LIST_TYPE);
            if (ObjectUtils.notEmpty(rules)) {
                rules.removeIf(rule -> {
                    if (this.matches(rule)) {
                        log.info("Applying firewall rule: {}", rule);
                        return false;
                    }
                    return true;
                });
            }
            return ObjectUtils.isEmpty(rules) ? Collections.emptyList() : rules;
        });
    }

    /**
     * 获取IP白名单
     *
     * @return IP列表
     */
    private List<Ip> getWhitelists() {
        return ApplicationContextHolder.getProperty("security.firewall.whitelists", Ip::host2ips);
    }

    /**
     * 获取IP黑名单
     *
     * @return IP列表
     */
    private List<Ip> getBlacklists() {
        return ApplicationContextHolder.getProperty("security.firewall.blacklists", Ip::host2ips);
    }

    /**
     * 判断规则是否匹配
     *
     * @param rule 规则信息
     * @return true/false
     */
    private boolean matches(Rule rule) {
        return (StringUtils.isEmpty(rule.getApp())
                || rule.getApp().equalsIgnoreCase(ApplicationContextHolder.getApplicationName()))
                && StringUtils.notEmpty(rule.getAction()) && rule.getCondition() != null
                && (rule.getCount() < 1 || (rule.getPrincipal() != null
                && rule.getCycle() > 0 && rule.getCycle() <= 3600
                && rule.getExpiration() >= 60 && rule.getExpiration() <= 1800));
    }

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler)
            throws Exception {
        if (!(handler instanceof HandlerMethod) || !this.isEnabled()) {
            return true;
        }

        String host = SessionContextHolder.getHost();
        List<Ip> whitelists = this.getWhitelists(), blacklists = this.getBlacklists();
        if (StringUtils.notEmpty(host) && (ObjectUtils.notEmpty(whitelists) || ObjectUtils.notEmpty(blacklists))) {
            // 获取ip数字
            long number;
            try {
                number = Ip.getAddressNumber(host);
            } catch (Exception e) {
                log.error("Address number convert failed", e);
                return true;
            }

            // 如果是IP白名单则放行
            for (Ip ip : whitelists) {
                if (ip.matches(number)) {
                    return true;
                }
            }

            // 如果是IP黑明单则拒绝访问
            for (Ip ip : blacklists) {
                if (ip.matches(number)) {
                    throw new FrameworkException(HttpStatus.FORBIDDEN.name(), null,
                            ApplicationContextHolder.getMessage("Forbidden"));
                }
            }
        }

        // 规则防护处理
        List<Rule> rules = this.getRules();
        return ObjectUtils.isEmpty(rules) || this.processor.processing(rules);
    }

    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        registry.addInterceptor(this).addPathPatterns("/**").order(Ordered.HIGHEST_PRECEDENCE);
    }
}
